@@ -38,6 +38,9 @@ from igraph import *
 from cdlib import algorithms
 #from infomap import Infomap
 from networkx.algorithms import community
+import wordninja
+from langdetect import detect
+from googletrans import Translator
 
 
 #---------------------------------------------NLP-Part-------------------------------------------------------------------------------#
@@ -1735,3 +1738,81 @@ def df_graph_telegram_features (DiG, nodes_participants_df):
     df_graph_features['eigenvector_centrality'] =eigenvector_centrality
     df_graph_features["neighbours"] = neighbours_names
     return df_graph_features
+
+###----------------------Contextual Analysis Telegram -----------------------###
+
+def translateToEnglish(msg):
+    translator = Translator()
+    l=[]
+    for i in range(len(msg)):
+        if msg.language.iloc[i] == 'en':
+            l.append(msg.message.iloc[i])
+        elif msg.message.iloc[i] == None or msg.message.iloc[i] == '':
+            l.append('')
+        else:
+            translation = translator.translate( msg.message.iloc[i], dest='en')
+            l.append(translation.text)
+    msg['translated_message'] = l
+    return msg 
+
+def cosine_distance_countvectorizer_method(s1, s2):
+    
+    # sentences to list
+    allsentences = [s1 , s2]
+    
+    # packages
+    from sklearn.feature_extraction.text import CountVectorizer
+    from scipy.spatial import distance
+    
+    # text to vector
+    vectorizer = CountVectorizer()
+    all_sentences_to_vector = vectorizer.fit_transform(allsentences)
+    text_to_vector_v1 = all_sentences_to_vector.toarray()[0].tolist()
+    text_to_vector_v2 = all_sentences_to_vector.toarray()[1].tolist()
+    
+    # distance of sifomilarity
+    cosine = distance.cosine(text_to_vector_v1, text_to_vector_v2)
+    cosine_similarity = round((1-cosine),2)
+    #print('Similarity of two sentences are equal to ',round((1-cosine)*100,2),'%')
+    return cosine_similarity
+
+def content_similitarity(msg, corpus):
+    l=[]
+    for i in range(len(msg)):
+        l.append(cosine_distance_countvectorizer_method(msg.translated_message.iloc[i], corpus)*100)
+    msg['terrorist_content_similarity'] = l
+    return msg
+
+def detect_hastags(msg):
+    hasht=[]
+    for i in range(len(msg)):
+        if len(str(msg.message.iloc[i]).split('#'))>1:
+            hasht.append(str(msg.message.iloc[i]).split('#')[1].split(' ')[0])
+        else:
+            hasht.append('no hashtags')
+    msg['hashtags'] = hasht
+    return msg
+
+def hastags_preprocessing(msg): 
+    l=[]
+    for i in range(len(msg)):
+        x=( re.sub("[^a-zA-Z]", " ",str(msg.hashtags.iloc[i])))
+        x=re.sub(' +', ' ', x)
+        m=wordninja.split(x)
+        h=''
+        for j in range(len(m)):
+            h+= m[j]+' '
+        #print(h)
+        l.append(h.strip())
+    msg['hashtags'] = l
+    return msg
+
+def contextual_analysis_telegram(messages, corpus):
+    messages = detectLanguge(messages)
+    messages = translateToEnglish(messages)
+    messages = detect_hastags(messages)
+    messages = hastags_preprocessing(messages)
+    messages = content_similitarity(messages, corpus)
+    return messages
+
+   
